function S = SNRdB(pd,pfa,N,Swerling_case)

% Function SNRdB(pd,pfa,N,Swerling_case)
%
%
% S = SNRdB(pd,pfa,N,swerlingcase)
%   Calculates SNR in dB required to achieve a given Pd and Pfa for
%   non-coherent pulse integration of Swerling target models in Gaussian
%   I/Q noise with a square-law detector.  Builds on Pd.m, which in turn is
%   based on the equations in the appendix of "Radar Target Detection-
%   Handbook of Theory and Practice" by Daniel P. Meyer and Herbert A.
%   Mayer (M&M) (Academic Press, 1973) to calculate the  threshold and
%   probability of detection for the four standard Swerling cases and the
%   nonfluctuating case.  Also included are the Albersheim approximation to
%   the nonfluctuating case, and the Shnidman approximations to the
%   Swerling cases.
%
%   Inputs: N = number of pulses to noncoherently integrate (matrix)
%           pfa = probability of false alarm (matrix, same dimensions as N)
%           pd = probability of detection (matrix, same dimensions as N)
%           Swerling_case = 0 or 5 (no fluctuation), 1, 2, 3, 4 (Swerling cases),
%                           or 6 (Albersheim's equation) (scalar)
%
% Mark A. Richards, May 2010
%
% Updated to provde a closed-form solution for the Swerling 2 case, June
% 2014

[R,C] = size(N);

%
% Swerling Case 0 (or 5): Non-fluctuating target model
%
if ( Swerling_case == 0  | Swerling_case == 5 )
    
    % Initial estimate from Albersheim
    A = log(0.62./pfa);
    B = log(pd./(1-pd));
    S0 = -5*log10(N) + ...
        (6.2 + 4.54./sqrt(N+0.44)).*log10(A + 0.12*A.*B + 1.7*B);
    
    % Now use fzero to refine that estimate. Have to do this one element at
    % a time
    for r = 1:R
        for c = 1:C
            S(r,c) = fzero(@pdzero,S0(r,c), ...
                optimset('disp','off','TolX',0.001), ...
                N(r,c),pfa(r,c),Swerling_case,pd(r,c));
        end
    end
    
    %
    % Cases 1-4: Swerling Models.
    %
    % We have a closed form solution for Swerling 2, but the other three
    % Swerling cases require an iterative search for the value of SNR that
    % produces the specified Pd when Pfa and N are as specified.
    %
elseif ( Swerling_case == 1  |   ...
        Swerling_case == 3  | Swerling_case == 4 )
    
    for r = 1:R
        for c = 1:C
            %
            % S0 and S(r,c) are in dB
            %
            % Initial estimate from Shnidman
            S0 = 10*log10(shnidman(N(r,c),pd(r,c),pfa(r,c),Swerling_case));
            % Now refine it with fzero
            S(r,c) = fzero(@pdzero,S0, ...
                optimset('disp','off','TolX',0.001), ...
                N(r,c),pfa(r,c),Swerling_case,pd(r,c));
        end
    end
    
elseif ( Swerling_case == 2 )
    
    for r = 1:R
        for c = 1:C
            % S(r,c) is computed on a linear scale, must be converted to dB
            S(r,c) = gammaincinv(1-pfa(r,c),N(r,c))/gammaincinv(1-pd(r,c),N(r,c)) - 1;
            S(r,c) = 10*log10(S(r,c));
        end
    end
    
    
    %
    % Case 6: Albersheim's Equation for nonfluctuating, **linear** detector
    %
elseif (Swerling_case == 6 )
    A = log(0.62./pfa);
    B = log(pd./(1-pd));
    S = -5*log10(N) + ...
        (6.2 + 4.54./sqrt(N+0.44)).*log10(A + 0.12*A.*B + 1.7*B);
    
else
    error('SNRdB: Error: Invalid Swerling case.') ;
    
end