function SNR1 = shnidman(N,Pd,Pfa,Scase)

% shnidman.m
%
% Implements Shnidman's equation for approximating radar detection curves
%
% Output SNR1 is on a linear scale, not dB
%
% Result is accurate within 1 dB (almost always within 0.5 dB) for
% 0.1 <= Pd <= 0.99, 1e-9 <= Pfa <= 1e-3, and 1 <= N <= 100.
%
% N   = number of samples noncoherently integrated
% Pd  = probability of detection
% Pfa = probability of false alarm
% Scase = Swerling case
%        Scase = 1 --> Swerling 1
%        Scase = 2 --> Swerling 2
%        Scase = 3 --> Swerling 3
%        Scase = 4 --> Swerling 4
%        Scase = 0 or 5 --> Marcum ("Swerling 0" or "Swerling 5" or
%                          nonfluctuating)
%
% D. A. Shnidman, "Determination of Required SNR Values," IEEE Trans. AES,
% vol. 38(3), pp. 1059-1064, July 2002.
%
% This code implements the basic version of the equation. The same
% reference includes an modification that extends the upper range of validity
% from Pd = 0.99 to Pd = 0.9992 if we restrict N to N <= 20.  This
% extension is NOT implemented in this code.

switch Scase
    case 0
        K = inf;
    case 1
        K = 1;
    case 2
        K = N;
    case 3
        K = 2;
    case 4
        K = 2*N;
    case 5
        K = inf;
    otherwise
        disp(['Invalid Swerling case (Scase = ',num2str(Scase)])
        return
end

if N < 40
    alpha = 0;
else
    alpha = 0.25;
end

eta = sqrt(-0.8*log(4*Pfa*(1-Pfa))) + sign(Pd-0.5)*sqrt(-0.8*log(4*Pd*(1-Pd)));

Xinf = eta*(eta+2*sqrt(N/2+(alpha - 0.25)));

C1 = (((17.7006*Pd - 18.4496)*Pd + 14.5339)*Pd - 3.525)/K;
C2 = exp(27.31*Pd - 25.14)/K + (Pd - 0.8)*(0.7*log(1e-5/Pfa) + (2*N - 20)/80)/K;
if Pd <= 0.872
    CdB = C1;
else
    CdB = C1 + C2;
end
C = 10^(CdB/10);

Xap = Xinf*C;

SNR1 = Xap/N;
